// 001 u_c_xxx
// 04 mar 00

(*$r+*)

unit u_c_calcul_charges_sas;
  interface
    uses u_c_basic_object
        , u_c_sas_parameter_list_2
        ;

    type c_calcul_charges_sas=
             class(c_basic_object)
               m_c_sas_parameter_list_2_ref: c_sas_parameter_list_2;
               m_month_count: Integer;

               Constructor create_calcul_charges_sas(p_name: String;
                   p_c_sas_parameter_list_2_ref: c_sas_parameter_list_2);
               procedure compute_charges(p_salary: Double);
               Destructor Destroy; Override;
             end; // c_calcul_charges_sas

  implementation
    uses SysUtils, Math
        , u_c_display
        ;

    // -- c_calcul_charges_sas

    Constructor c_calcul_charges_sas.create_calcul_charges_sas(p_name: String;
        p_c_sas_parameter_list_2_ref: c_sas_parameter_list_2);
      begin
        Inherited create_basic_object(p_name);
        m_c_sas_parameter_list_2_ref:= p_c_sas_parameter_list_2_ref;
      end; // create_calcul_charges_sas

    procedure c_calcul_charges_sas.compute_charges(p_salary: Double);
      var l_total_employee, l_total_company: Double;
          l_plafond_secu: Double;
          l_prevoyance_pour_csg: Double;

      procedure _display(p_key, p_policy: String;
          p_base, p_employee_rate, p_employee_amout, p_company_rate, p_company_amount: Double);

        procedure _format(p_rate, p_value: Double; var pv_rate, pv_value: String);
          begin
            pv_rate:= ''; pv_value:= '';
            if p_rate<> 0
              then begin
                  pv_rate:= Format('%7.3f', [p_rate]);
                  pv_value:= Format('%10.2f', [p_value]);
                end;
          end; // _format

        var l_employee_rate, l_employee_amount: String;
            l_company_rate, l_company_amount: String;

        begin // _display
          _format(p_employee_rate, p_employee_amout, l_employee_rate, l_employee_amount);
          _format(p_company_rate, p_company_amount, l_company_rate, l_company_amount);

(*
          display(Format('%-31s %5s %10.2f %6s %10s   %6s %10s',
               [p_key, p_policy, p_base, p_employee_rate, p_employee_amout, p_company_rate, p_company_amount]));
*)
          display(Format('%-27s %-5s %10.2f   %7s %10s     %7s %10s',
               [p_key, p_policy, p_base, l_employee_rate, l_employee_amount, l_company_rate, l_company_amount]));

          l_total_employee:= l_total_employee+ p_employee_amout;
          l_total_company:= l_total_company+ p_company_amount;
          if p_key= 'prevoyance_cadre'
            then l_prevoyance_pour_csg:= l_prevoyance_pour_csg+ p_company_amount;
        end; // _display

      procedure _compute_on_full_salary(p_c_sas_parameter: c_sas_parameter);
        var l_employee_amount, l_company_amount: Double;
        begin
          with p_c_sas_parameter do
          begin
            l_employee_amount:= p_salary* m_employee_rate/ 100;
            l_company_amount:= p_salary* m_company_rate/ 100;
            _display(m_name, m_policy, p_salary,
                m_employee_rate, l_employee_amount, m_company_rate, l_company_amount);
          end;
        end; // _compute_on_full_salary

      procedure _compute_on_plafond_secu(p_c_sas_parameter: c_sas_parameter);
        var l_base: Double;
            l_employee_amount, l_company_amount: Double;
        begin
          with p_c_sas_parameter do
          begin
            l_base:= Min(l_plafond_secu, p_salary);
            l_employee_amount:= l_base* m_employee_rate/ 100;
            l_company_amount:= l_base* m_company_rate/ 100;
            _display(m_name, m_policy, l_base,
                m_employee_rate, l_employee_amount, m_company_rate, l_company_amount);
          end;
        end; // _compute_on_plafond_secu

      procedure _compute_retraite_1_a_8_plafond(p_c_sas_parameter: c_sas_parameter);
        var l_base: Double;
            l_employee_amount, l_company_amount: Double;
        begin
          with p_c_sas_parameter do
          begin
            // display(Format('%10.f %10.2f', [p_salary, l_plafond_secu]));
            l_base:= Min(l_plafond_secu* 8, p_salary);
            // display(Format('%10.f', [l_base]));
            l_base:= l_base- l_plafond_secu;
            // display(Format('%10.f', [l_base]));
            if l_base> 0
              then begin
                  l_employee_amount:= l_base* m_employee_rate/ 100;
                  l_company_amount:= l_base* m_company_rate/ 100;
                  _display(m_name, m_policy, l_base,
                      m_employee_rate, l_employee_amount, m_company_rate, l_company_amount);
                end;
          end;
        end; // _compute_retraite_1_a_8_plafond

      procedure _compute_on_total_up_to_4_plafond(p_c_sas_parameter: c_sas_parameter);
        var l_base: Double;
            l_employee_amount, l_company_amount: Double;
        begin
          with p_c_sas_parameter do
          begin
            l_base:= Min(p_salary, l_plafond_secu* 4);
            l_employee_amount:= l_base* m_employee_rate/ 100;
            l_company_amount:= l_base* m_company_rate/ 100;
            // display(Format('%10.2f%10.5f %10.2f', [l_base, m_employee_rate, l_employee_amount]));
            _display(m_name, m_policy, l_base,
                m_employee_rate, l_employee_amount, m_company_rate, l_company_amount);
          end;
        end; // _compute_on_total_up_to_4_plafond

      procedure _compute_on_total_up_to_8_plafond(p_c_sas_parameter: c_sas_parameter);
        var l_base: Double;
            l_employee_amount, l_company_amount: Double;
        begin
          with p_c_sas_parameter do
          begin
            l_base:= Min(p_salary, l_plafond_secu* 8);
            l_employee_amount:= l_base* m_employee_rate/ 100;
            l_company_amount:= l_base* m_company_rate/ 100;
            _display(m_name, m_policy, l_base,
                m_employee_rate, l_employee_amount, m_company_rate, l_company_amount);
          end;
        end; // _compute_on_total_up_to_8_plafond

      procedure _compute_on_total_1_to_8_plafond(p_c_sas_parameter: c_sas_parameter);
        var l_base: Double;
            l_employee_amount, l_company_amount: Double;
        begin
          with p_c_sas_parameter do
          begin
            l_base:= Min(p_salary, l_plafond_secu* 8)- l_plafond_secu;
            if l_base> 0
              then begin
                  l_employee_amount:= l_base* m_employee_rate/ 100;
                  l_company_amount:= l_base* m_company_rate/ 100;
                  _display(m_name, m_policy, l_base,
                      m_employee_rate, l_employee_amount, m_company_rate, l_company_amount);
                end;
          end;
        end; // _compute_on_total_1_to_8_plafond

      procedure _compute_csg(p_c_sas_parameter: c_sas_parameter);
        var l_max, l_base: Double;
            l_employee_amount: Double;
        begin
          with p_c_sas_parameter do
          begin
            l_base:= Min(p_salary, l_plafond_secu* 4)* 0.9825;
            if p_salary> l_plafond_secu* 4
              then l_base:= l_base+ (p_salary- l_plafond_secu* 4);

            l_base:= l_base+ l_prevoyance_pour_csg;

            l_employee_amount:= p_salary* m_employee_rate/ 100;
            _display(m_name, m_policy, l_base,
                m_employee_rate, l_employee_amount, 0, 0);
          end;
        end; // _compute_csg





      procedure _compute_xxx(p_c_sas_parameter: c_sas_parameter);
        var l_employee_amount, l_company_amount: Double;
        begin
          with p_c_sas_parameter do
          begin
            l_employee_amount:= p_salary* m_employee_rate/ 100;
            l_company_amount:= p_salary* m_company_rate/ 100;
            _display(m_name, m_policy, p_salary,
                m_employee_rate, l_employee_amount, m_company_rate, l_company_amount);
          end;
        end; // _compute_xxx

      var l_sas_parameter_index: Integer;

      begin // compute_charges
        l_total_employee:= 0; l_total_company:= 0;
        l_prevoyance_pour_csg:= 0;

        display_line;
        display(Format('--- Charges SAS. Salaire %10.2f', [p_salary]));

        with m_c_sas_parameter_list_2_ref do
        begin
          l_plafond_secu:= f_c_find_sas_parameter_by_name('plafond_secu').m_employee_rate;

          for l_sas_parameter_index:= 0 to f_sas_parameter_count- 1 do
            with f_c_sas_parameter(l_sas_parameter_index) do
              if m_policy= 'T'
                then _compute_on_full_salary(f_c_self) else
              if m_policy= 'P'
                then _compute_on_plafond_secu(f_c_self) else
              if m_policy= 'B_1_8'
                then _compute_retraite_1_a_8_plafond(f_c_self) else
              if m_policy= 'T_<_4'
                then _compute_on_total_up_to_4_plafond(f_c_self) else
              if m_policy= 'T_1_8'
                then _compute_on_total_1_to_8_plafond(f_c_self) else
              if m_policy= 'T_<_8'
                then _compute_on_total_up_to_8_plafond(f_c_self) else
              if m_policy= 'CSG'
                then _compute_csg(f_c_self)
                else
                  if m_policy= ''
                    then display_line
                    else  _display(m_name, m_policy, p_salary, 0, 0, 0, 0);
        end; // with m_c_sas_parameter_list_2_ref

        display_line;
        display(Format('total %s      %6.2f  %10.2f %s   %6.2f  %10.2f',
            [f_spaces(10+ 25), l_total_employee/ p_salary*100, l_total_employee,
             f_spaces(1), l_total_company/ p_salary* 100, l_total_company]));

        display_line;
        display(Format('overall %s      %10.2f  %10.2f    %6.2f',
            [f_spaces(3), p_salary,
             l_total_employee+ l_total_company,
             (l_total_employee+ l_total_company)/ p_salary* 100]));
      end; // compute_charges

    Destructor c_calcul_charges_sas.Destroy;
      begin
        Inherited;
      end; // Destroy

    begin // u_c_calcul_charges_sas
    end. // u_c_calcul_charges_sas


